﻿using UnityEngine;
using HarmonyLib;
using System.Collections.Generic;

public class StackablePlaceables : Mod
{
    private Harmony harmony;
    public static bool overrideAccept = false;
    public static bool overrideRayhit = false;
    public static Vector3 rotation = Vector3.zero;
    static public JsonModInfo modInfo;
    public void Start()
    {
        modInfo = modlistEntry.jsonmodinfo;
        harmony = new Harmony("com.aidanamite.StackablePlaceables");
        harmony.PatchAll();
        Log("Mod has been loaded!");
    }

    [ConsoleCommand(name: "toggleForceAccept", docs: "When active, ignores most build location restrictions")]
    public static string MyCommand(string[] args)
    {
        overrideAccept = !overrideAccept;
        return "[" + modInfo.name + "]: Is " + (overrideAccept ? "now" : "no longer") + " overriding acceptable build location";
    }

    [ConsoleCommand(name: "toggleOverrideRaycast", docs: "When active, uses alternate placement location finder")]
    public static string MyCommand2(string[] args)
    {
        overrideRayhit = !overrideRayhit;
        if (!overrideRayhit)
            Patch_raycast.ResetPositions();
        return "[" + modInfo.name + "]: Is " + (overrideRayhit ? "now" : "no longer") + " overriding build location raycast";
    }

    [ConsoleCommand(name: "setBuildRotations", docs: "Syntax: setBuildRotations <xRot> <zRot> - Recommended to use multiples of 90")]
    public static string MyCommand3(string[] args)
    {
        if (args.Length < 2)
            return "Not enough arguments";
        if (args.Length > 2)
            return "Too many arguments";
        float x;
        try
        {
            x = float.Parse(args[0]);
        } catch
        {
            return "[" + modInfo.name + "]: " + args[0] + " cannot be parsed";
        }
        try
        {
            rotation = new Vector3(x,0,float.Parse(args[1]));
            return "[" + modInfo.name + "]: Rotations modified";
        }
        catch
        {
            return "[" + modInfo.name + "]: " + args[1] + " cannot be parsed";
        }
    }

    public static void Log(object message)
    {
        Debug.Log("[" + modInfo.name + "]: " + message.ToString());
    }

    public void Update()
    {
        if (Input.GetKey(MyInput.Keybinds["Rotate"].MainKey))
        {
            //Log("Holding R");
            if (Input.GetKeyDown(KeyCode.LeftArrow))
                rotation.x += 90;
            if (Input.GetKeyDown(KeyCode.RightArrow))
                rotation.x -= 90;
            if (Input.GetKeyDown(KeyCode.UpArrow))
                rotation.z += 90;
            if (Input.GetKeyDown(KeyCode.DownArrow))
                rotation.z -= 90;
            if (Input.GetKeyDown(KeyCode.LeftAlt) || Input.GetKeyDown(KeyCode.RightAlt))
                rotation = Vector3.zero;
            rotation.x %= 360;
            rotation.z %= 360;
        }
        if (MyInput.GetButton("Sprint"))
        {
            //Log("Holding S");
            if (Input.GetKeyDown(KeyCode.LeftArrow))
                overrideRayhit = true;
            if (Input.GetKeyDown(KeyCode.RightArrow))
                overrideRayhit = false;
            if (Input.GetKeyDown(KeyCode.UpArrow))
                overrideAccept = true;
            if (Input.GetKeyDown(KeyCode.DownArrow))
                overrideAccept = false;
            if (Input.GetKeyDown(KeyCode.LeftAlt) || Input.GetKeyDown(KeyCode.RightAlt))
            {
                overrideRayhit = false;
                overrideAccept = false;
            }
            if (!overrideRayhit)
                Patch_raycast.ResetPositions();
        }
        
    }

    public void OnModUnload()
    {
        harmony.UnpatchAll();
        Patch_raycast.ResetPositions();
        Log("Mod has been unloaded!");
    }
}

[HarmonyPatch(typeof(BlockQuad), "AcceptsBlock")]
public class Patch_accepter
{
    static void Postfix(ref bool __result)
    {
        if (StackablePlaceables.overrideAccept)
            __result = true;
    }
}

[HarmonyPatch(typeof(BlockCreator), "GetQuadAtCursor")]
public class Patch_raycast
{
    static List<BlockQuad> modified = new List<BlockQuad>();
    static List<Vector3> originalPositions = new List<Vector3>();
    static void Postfix(ref BlockCreator __instance, ref BlockQuad __result)
    {
        if (StackablePlaceables.overrideRayhit)
        {
            RaycastHit raycastHit;
            if (__result != null && (Helper.HitAtCursor(out raycastHit, Player.UseDistance, LayerMasks.MASK_Item) || Helper.HitAtCursor(out raycastHit, Player.UseDistance, LayerMasks.MASK_RaycastInteractable)))
            {
                if (!modified.Contains(__result))
                {
                    modified.Add(__result);
                    originalPositions.Add(__result.transform.localPosition);
                }
                __result.transform.position = raycastHit.point;
            }
        }
    }
    public static void ResetPositions()
    {
        for (int i = 0;i < modified.Count;i++)
            if (modified[i].IsNotNull())
                modified[i].transform.localPosition = originalPositions[i];
        modified.Clear();
        originalPositions.Clear();
    }
}

[HarmonyPatch(typeof(BlockCreator), "SetGhostBlockPositionAndRotation")]
public class Patch_Rotation
{
    static void Postfix(ref BlockCreator __instance)
    {
        __instance.selectedBlock.transform.localEulerAngles += StackablePlaceables.rotation;
    }
}

[HarmonyPatch(typeof(BlockCreator), "SnapBuildRotation")]
public class Patch_CheckRotation
{
    static bool Prefix()
    {
        return false;
    }
}

[HarmonyPatch(typeof(BlockCreator), "CanBuildBlock")]
public class Patch_CheckBuildError
{
    static void Postfix(ref BuildError __result)
    {
        if ((Input.GetKey(KeyCode.LeftAlt) || Input.GetKey(KeyCode.RightAlt)) && (__result == BuildError.PositionOccupied || __result == BuildError.PositionOccupied_Same || __result == BuildError.InWater || (__result == BuildError.NotStable && MyInput.GetButton("Sprint"))))
            __result = BuildError.None;
    }
}
